from typing import Dict, Any

from fastapi import HTTPException, status
from fastapi.routing import APIRouter
from orchestrator.services import processes, subscriptions
from pydantic import BaseModel

from gso.products.product_blocks.site import SiteTier

router = APIRouter()


class SiteImport(BaseModel):
    site_name: str
    site_city: str
    site_country: str
    site_country_code: str
    site_latitude: float
    site_longitude: float
    site_bgp_community_id: int
    site_internal_id: int
    site_tier: SiteTier
    site_ts_address: str


@router.post("/", status_code=status.HTTP_201_CREATED, tags=["Import"])
def import_site(site: SiteImport) -> Dict[str, Any]:
    """
    Import site by running the import_site workflow.
    response:
        - pid: The process id of the started process.
    Raises:
    HTTPException: If the site already exists or if there's an error in the process.
    """
    subscription = subscriptions.retrieve_subscription_by_subscription_instance_value(
        resource_type="site_name", value=site.site_name, sub_status=("provisioning", "active"))
    if subscription:
        raise HTTPException(status_code=status.HTTP_409_CONFLICT, detail="Site already exists.")

    pid = processes.start_process("import_site", [site.dict()])
    if pid is None:
        raise HTTPException(status_code=status.HTTP_500_INTERNAL_SERVER_ERROR, detail="Failed to start the process.")

    process = processes._get_process(pid)  # pylint: disable=protected-access
    if process.last_status == "failed":
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=f"Process {pid} failed because of an internal error.")

    return {"pid": str(pid)}
